/*
 * eset_rtp (ESET Real-time file system protection module)
 * Copyright (C) 1992-2021 ESET, spol. s r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * In case of any questions, you can contact us at ESET, spol. s r.o., Einsteinova 24, 851 01 Bratislava, Slovakia.
 */

#include "ertp.h"

#include <linux/moduleparam.h>

/* Module parameter macros */
static unsigned long sys_table_32 = 0;
module_param(sys_table_32, ulong, 0);
MODULE_PARM_DESC(sys_table_32, " Base address of the x86 syscall table");

static unsigned long sys_table_64 = 0;
module_param(sys_table_64, ulong, 0);
MODULE_PARM_DESC(sys_table_64, " Base address of the x64 syscall table");

static int __init ertp_init(void)
{
	int rv = 0;

	if ((rv = ertp_cache_init()) != 0)
		return rv;

	if ((rv = ertp_events_init()) != 0)
		goto err_cache;

	if ((rv = ertp_hooks_init((uintptr_t)sys_table_32, (uintptr_t)sys_table_64)) != 0)
		goto err_events;

	if ((rv = ertp_sysfs_init()) != 0)
		goto err_hooks;

	if ((rv = ertp_dev_init()) != 0)
		goto err_sysfs;

	printk(KERN_INFO ESET_RTP_FULL_NAME " " ERTP_VERSION " <www.eset.com>\n");

	return 0;

err_sysfs:
	ertp_sysfs_exit();
err_hooks:
	ertp_hooks_exit();
err_events:
	ertp_events_exit();
err_cache:
	ertp_cache_exit();

	return rv;
}

static void __exit ertp_exit(void)
{
	ertp_dev_exit();
	ertp_sysfs_exit();
	ertp_hooks_exit();
	ertp_cache_exit(); /* cache contains event references, need to be destroyed before events */
	ertp_events_exit();
}

module_init(ertp_init);
module_exit(ertp_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Jozef Melicher <jozef.melicher@eset.sk>");
MODULE_AUTHOR("Viliam Simovic <simovic@eset.sk>");
MODULE_DESCRIPTION(ESET_RTP_FULL_NAME);
