/*
 * eset_rtp (ESET Real-time file system protection module)
 * Copyright (C) 1992-2021 ESET, spol. s r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * In case of any questions, you can contact us at ESET, spol. s r.o., Einsteinova 24, 851 01 Bratislava, Slovakia.
 */

#include "ertp.h"

struct ertp_path *ertp_path_alloc(const char* path_name)
{
	struct ertp_path *path;

	if (path_name == NULL || path_name[0] == '\0')
		return ERR_PTR(-EINVAL);

	path = kzalloc(sizeof(struct ertp_path), GFP_KERNEL);
	if (!path)
		return ERR_PTR(-ENOMEM);

	INIT_LIST_HEAD(&path->list);

	path->path_name = kmalloc(strlen(path_name) +1 , GFP_KERNEL);
	if (!path->path_name) {
		kfree(path);
		return ERR_PTR(-ENOMEM);
	}

	memcpy(path->path_name, path_name, strlen(path_name) +1);
	path->path_len = strlen(path_name);
	atomic_set(&path->count, 1);

	return path;
}

struct ertp_path *ertp_path_get(struct ertp_path *path)
{
	if (!path || IS_ERR(path))
		return NULL;

	BUG_ON(!atomic_read(&path->count));
	atomic_inc(&path->count);

	return path;
}

void ertp_path_put(struct ertp_path *path)
{
	if (!path || IS_ERR(path))
		return;

	BUG_ON(!atomic_read(&path->count));
	if (!atomic_dec_and_test(&path->count))
		return;

	if (path->path_name)
		kfree(path->path_name);

	kfree(path);
}
