/*
 * eset_rtp (ESET Real-time file system protection module)
 * Copyright (C) 1992-2021 ESET, spol. s r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * In case of any questions, you can contact us at ESET, spol. s r.o., Einsteinova 24, 851 01 Bratislava, Slovakia.
 */

#ifdef CONFIG_X86_X32

#include <linux/version.h>

#include "../arch/x86/include/uapi/asm/unistd.h"
#include "../arch/x86/include/generated/uapi/asm/unistd_x32.h"

#include "ertp_syscalls.h"

#define GET_SYSCALL_NO(x) ((x) & ~(__X32_SYSCALL_BIT))

const int ERTP_SYSCALL_NUMBERS_X32[N_ERTP_SYSCALLS] = {
	GET_SYSCALL_NO(__NR_open),       /* ERTP_SYSCALL_open */
	GET_SYSCALL_NO(__NR_openat),     /* ERTP_SYSCALL_openat */
	GET_SYSCALL_NO(__NR_close),      /* ERTP_SYSCALL_close */
	GET_SYSCALL_NO(__NR_exit),       /* ERTP_SYSCALL_exit */
	GET_SYSCALL_NO(__NR_exit_group), /* ERTP_SYSCALL_exit_group */
	GET_SYSCALL_NO(__NR_execve),     /* ERTP_SYSCALL_execve */
	GET_SYSCALL_NO(__NR_dup2),       /* ERTP_SYSCALL_dup2 */
	GET_SYSCALL_NO(__NR_dup3),       /* ERTP_SYSCALL_dup3 */
	GET_SYSCALL_NO(__NR_unlink),     /* ERTP_SYSCALL_unlink */
	GET_SYSCALL_NO(__NR_unlinkat),   /* ERTP_SYSCALL_unlinkat */
	GET_SYSCALL_NO(__NR_rename),     /* ERTP_SYSCALL_rename */
	GET_SYSCALL_NO(__NR_renameat),   /* ERTP_SYSCALL_renameat */
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,15,0))
	GET_SYSCALL_NO(__NR_renameat2),  /* ERTP_SYSCALL_renameat2 */
#endif
	GET_SYSCALL_NO(__NR_mmap),      /* ERTP_SYSCALL_mmap */
};

#endif /* CONFIG_X86_X32 */
