/*
 * eset_rtp (ESET Real-time file system protection module)
 * Copyright (C) 1992-2021 ESET, spol. s r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * In case of any questions, you can contact us at ESET, spol. s r.o., Einsteinova 24, 851 01 Bratislava, Slovakia.
 */

#include "ertp.h"

#ifdef ERTP_DEBUG
char *ertp_getname(const char __user *filename)
{
	long len;
	char *result = __getname();
	if (unlikely(!result))
		return ERR_PTR(-ENOMEM);

	len = strncpy_from_user(result, filename, PATH_MAX);
	if (unlikely(len < 0)) {
		__putname(result);
		return ERR_PTR(len);
	}
	if (unlikely(len == PATH_MAX)) {
		__putname(result);
		return ERR_PTR(-ENAMETOOLONG);
	}

	return result;
}
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,7,0)
unsigned long ertp_lookup_address(const char *symbol_name)
{
	struct kprobe probe = {
		.symbol_name = symbol_name
	};
	unsigned long ret;

	if (register_kprobe(&probe) < 0) {
		return 0;
	}

	ret = (unsigned long) probe.addr;
	unregister_kprobe(&probe);
	return ret;
}
#else // LINUX_VERSION_CODE >= KERNEL_VERSION(5,7,0)
unsigned long ertp_lookup_address(const char *symbol_name)
{
	return kallsyms_lookup_name(symbol_name);
}
#endif
